@@ -28,39 +28,47 @@
 
 #include "neochat-version.h"
 
-#ifdef QUOTIENT_07
-#include <accountregistry.h>
-#else
-#include "neochataccountregistry.h"
-#endif
-
-#include <networkaccessmanager.h>
-#include <room.h>
-#include <util.h>
+#include <Quotient/keyverificationsession.h>
+#include <Quotient/accountregistry.h>
+#include <Quotient/networkaccessmanager.h>
+#include <Quotient/room.h>
+#include <Quotient/util.h>
 
 #include "actionshandler.h"
 #include "blurhashimageprovider.h"
 #include "chatdocumenthandler.h"
 #include "clipboard.h"
 #include "controller.h"
+#include "delegatesizehelper.h"
 #include "filetypesingleton.h"
-#include "joinrulesevent.h"
 #include "linkpreviewer.h"
+#include "locationhelper.h"
+#include "logger.h"
 #include "login.h"
 #include "matriximageprovider.h"
+#include "models/accountemoticonmodel.h"
 #include "models/collapsestateproxymodel.h"
 #include "models/customemojimodel.h"
 #include "models/devicesmodel.h"
+#include "models/devicesproxymodel.h"
 #include "models/emojimodel.h"
-#include "models/keywordnotificationrulemodel.h"
+#include "models/emoticonfiltermodel.h"
+#include "models/imagepacksmodel.h"
+#include "models/livelocationsmodel.h"
+#include "models/locationsmodel.h"
+#include "models/mediamessagefiltermodel.h"
 #include "models/messageeventmodel.h"
 #include "models/messagefiltermodel.h"
 #include "models/publicroomlistmodel.h"
+#include "models/pushrulemodel.h"
+#include "models/reactionmodel.h"
 #include "models/roomlistmodel.h"
 #include "models/searchmodel.h"
 #include "models/serverlistmodel.h"
 #include "models/sortfilterroomlistmodel.h"
 #include "models/sortfilterspacelistmodel.h"
+#include "models/statefiltermodel.h"
+#include "models/stickermodel.h"
 #include "models/userdirectorylistmodel.h"
 #include "models/userfiltermodel.h"
 #include "models/userlistmodel.h"
@@ -69,16 +77,12 @@
 #include "neochatroom.h"
 #include "neochatuser.h"
 #include "notificationsmanager.h"
-#ifdef QUOTIENT_07
 #include "pollhandler.h"
-#endif
 #include "roommanager.h"
 #include "spacehierarchycache.h"
 #include "urlhelper.h"
 #include "windowcontroller.h"
-#ifdef QUOTIENT_07
-#include <keyverificationsession.h>
-#endif
+
 #ifdef HAVE_COLORSCHEME
 #include "colorschemer.h"
 #endif
@@ -118,7 +122,9 @@ Q_DECL_EXPORT
 #endif
 int main(int argc, char *argv[])
 {
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     QGuiApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
+#endif
 
     QNetworkProxyFactory::setUseSystemConfiguration(true);
 
@@ -155,7 +161,7 @@ int main(int argc, char *argv[])
                      QStringLiteral(NEOCHAT_VERSION_STRING),
                      i18n("Matrix client"),
                      KAboutLicense::GPL_V3,
-                     i18n("© 2018-2020 Black Hat, 2020-2022 KDE Community"));
+                     i18n("© 2018-2020 Black Hat, 2020-2023 KDE Community"));
     about.addAuthor(i18n("Carl Schwan"), i18n("Maintainer"), QStringLiteral("carl@carlschwan.eu"), QStringLiteral("https://carlschwan.eu"));
     about.addAuthor(i18n("Tobias Fella"), i18n("Maintainer"), QStringLiteral("tobias.fella@kde.org"), QStringLiteral("https://tobiasfella.de"));
     about.addAuthor(i18n("James Graham"), i18n("Maintainer"), QStringLiteral("james.h.graham@protonmail.com"));
@@ -166,20 +172,22 @@ int main(int argc, char *argv[])
 
     about.addComponent(QStringLiteral("libQuotient"),
                        i18n("A Qt5 library to write cross-platform clients for Matrix"),
-#ifdef QUOTIENT_07
                        i18nc("<version number> (built against <possibly different version number>)",
                              "%1 (built against %2)",
                              Quotient::versionString(),
                              QStringLiteral(Quotient_VERSION_STRING)),
-#else
-                       QStringLiteral(QUOTIENT_VERSION),
-#endif
                        QStringLiteral("https://github.com/quotient-im/libquotient"),
                        KAboutLicense::LGPL_V2_1);
 
     KAboutData::setApplicationData(about);
     QGuiApplication::setWindowIcon(QIcon::fromTheme(QStringLiteral("org.kde.neochat")));
 
+    initLogging();
+
+#if Quotient_VERSION_MINOR == 8
+    Connection::setEncryptionDefault(true);
+#endif
+
 #ifdef NEOCHAT_FLATPAK
     // Copy over the included FontConfig configuration to the
     // app's config dir:
@@ -210,11 +218,7 @@ int main(int argc, char *argv[])
     qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "LoginHelper", login);
     qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "UrlHelper", &urlHelper);
     qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "EmojiModel", &EmojiModel::instance());
-#ifdef QUOTIENT_07
-    qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "AccountRegistry", &Quotient::Accounts);
-#else
-    qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "AccountRegistry", &Quotient::AccountRegistry::instance());
-#endif
+    qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "AccountRegistry", &Controller::instance().accounts());
     qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "SpaceHierarchyCache", &SpaceHierarchyCache::instance());
     qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "CustomEmojiModel", &CustomEmojiModel::instance());
     qmlRegisterType<ActionsHandler>("org.kde.neochat", 1, 0, "ActionsHandler");
@@ -223,7 +227,9 @@ int main(int argc, char *argv[])
     qmlRegisterType<KWebShortcutModel>("org.kde.neochat", 1, 0, "WebShortcutModel");
     qmlRegisterType<UserListModel>("org.kde.neochat", 1, 0, "UserListModel");
     qmlRegisterType<MessageEventModel>("org.kde.neochat", 1, 0, "MessageEventModel");
+    qmlRegisterType<ReactionModel>("org.kde.neochat", 1, 0, "ReactionModel");
     qmlRegisterType<CollapseStateProxyModel>("org.kde.neochat", 1, 0, "CollapseStateProxyModel");
+    qmlRegisterType<MediaMessageFilterModel>("org.kde.neochat", 1, 0, "MediaMessageFilterModel");
     qmlRegisterType<MessageFilterModel>("org.kde.neochat", 1, 0, "MessageFilterModel");
     qmlRegisterType<UserFilterModel>("org.kde.neochat", 1, 0, "UserFilterModel");
     qmlRegisterType<PublicRoomListModel>("org.kde.neochat", 1, 0, "PublicRoomListModel");
@@ -232,20 +238,29 @@ int main(int argc, char *argv[])
     qmlRegisterType<SortFilterRoomListModel>("org.kde.neochat", 1, 0, "SortFilterRoomListModel");
     qmlRegisterType<SortFilterSpaceListModel>("org.kde.neochat", 1, 0, "SortFilterSpaceListModel");
     qmlRegisterType<DevicesModel>("org.kde.neochat", 1, 0, "DevicesModel");
+    qmlRegisterType<DevicesProxyModel>("org.kde.neochat", 1, 0, "DevicesProxyModel");
     qmlRegisterType<LinkPreviewer>("org.kde.neochat", 1, 0, "LinkPreviewer");
     qmlRegisterType<CompletionModel>("org.kde.neochat", 1, 0, "CompletionModel");
     qmlRegisterType<StateModel>("org.kde.neochat", 1, 0, "StateModel");
+    qmlRegisterType<StateFilterModel>("org.kde.neochat", 1, 0, "StateFilterModel");
     qmlRegisterType<SearchModel>("org.kde.neochat", 1, 0, "SearchModel");
-#ifdef QUOTIENT_07
+    qmlRegisterType<LiveLocationsModel>("org.kde.neochat", 1, 0, "LiveLocationsModel");
+    qmlRegisterType<LocationsModel>("org.kde.neochat", 1, 0, "LocationsModel");
     qmlRegisterType<PollHandler>("org.kde.neochat", 1, 0, "PollHandler");
-#endif
-    qmlRegisterType<KeywordNotificationRuleModel>("org.kde.neochat", 1, 0, "KeywordNotificationRuleModel");
+    qmlRegisterType<PushRuleModel>("org.kde.neochat", 1, 0, "PushRuleModel");
+    qmlRegisterType<StickerModel>("org.kde.neochat", 1, 0, "StickerModel");
+    qmlRegisterType<ImagePacksModel>("org.kde.neochat", 1, 0, "ImagePacksModel");
+    qmlRegisterType<AccountEmoticonModel>("org.kde.neochat", 1, 0, "AccountEmoticonModel");
+    qmlRegisterType<EmoticonFilterModel>("org.kde.neochat", 1, 0, "EmoticonFilterModel");
+    qmlRegisterType<DelegateSizeHelper>("org.kde.neochat", 1, 0, "DelegateSizeHelper");
     qmlRegisterUncreatableType<RoomMessageEvent>("org.kde.neochat", 1, 0, "RoomMessageEvent", "ENUM");
+    qmlRegisterUncreatableType<PushNotificationKind>("org.kde.neochat", 1, 0, "PushNotificationKind", "ENUM");
+    qmlRegisterUncreatableType<PushNotificationSection>("org.kde.neochat", 1, 0, "PushNotificationSection", "ENUM");
     qmlRegisterUncreatableType<PushNotificationState>("org.kde.neochat", 1, 0, "PushNotificationState", "ENUM");
     qmlRegisterUncreatableType<PushNotificationAction>("org.kde.neochat", 1, 0, "PushNotificationAction", "ENUM");
     qmlRegisterUncreatableType<NeoChatRoomType>("org.kde.neochat", 1, 0, "NeoChatRoomType", "ENUM");
-    qmlRegisterUncreatableType<UserType>("org.kde.neochat", 1, 0, "UserType", "ENUM");
     qmlRegisterUncreatableType<NeoChatUser>("org.kde.neochat", 1, 0, "NeoChatUser", {});
+    qmlRegisterUncreatableType<NeoChatRoom>("org.kde.neochat", 1, 0, "NeoChatRoom", {});
 
     qRegisterMetaType<User *>("User*");
     qRegisterMetaType<User *>("const User*");
@@ -257,16 +272,18 @@ int main(int argc, char *argv[])
     qRegisterMetaType<NeoChatUser *>("NeoChatUser*");
     qRegisterMetaType<GetRoomEventsJob *>("GetRoomEventsJob*");
     qRegisterMetaType<QMimeType>("QMimeType");
-#ifdef QUOTIENT_07
 #ifdef Quotient_E2EE_ENABLED
     qRegisterMetaType<KeyVerificationSession *>("KeyVerificationSession*");
     qmlRegisterUncreatableType<KeyVerificationSession>("org.kde.neochat", 1, 0, "KeyVerificationSession", {});
     qRegisterMetaType<QVector<EmojiEntry>>("QVector<EmojiEntry>");
-#endif
 #endif
     qmlRegisterSingletonType("org.kde.neochat", 1, 0, "About", [](QQmlEngine *engine, QJSEngine *) -> QJSValue {
         return engine->toScriptValue(KAboutData::applicationData());
     });
+    qmlRegisterSingletonType(QUrl("qrc:/OsmLocationPlugin.qml"), "org.kde.neochat", 1, 0, "OsmLocationPlugin");
+    qmlRegisterSingletonType("org.kde.neochat", 1, 0, "LocationHelper", [](QQmlEngine *engine, QJSEngine *) -> QJSValue {
+        return engine->toScriptValue(LocationHelper());
+    });
 
 #if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     qRegisterMetaTypeStreamOperators<Emoji>();
