@@ -2,17 +2,17 @@
 // SPDX-License-Identifier: LGPL-2.0-or-later
 
 #include "collapsestateproxymodel.h"
-#include "messageeventmodel.h"
 
 #include <KLocalizedString>
 
 bool CollapseStateProxyModel::filterAcceptsRow(int source_row, const QModelIndex &source_parent) const
 {
     Q_UNUSED(source_parent);
-    return sourceModel()->data(sourceModel()->index(source_row, 0), MessageEventModel::EventTypeRole)
+    return sourceModel()->data(sourceModel()->index(source_row, 0), MessageEventModel::DelegateTypeRole)
         != MessageEventModel::DelegateType::State // If this is not a state, show it
-        || sourceModel()->data(sourceModel()->index(source_row + 1, 0), MessageEventModel::EventTypeRole)
-        != MessageEventModel::DelegateType::State // If this is the first state in a block, show it. TODO hidden events?
+        || (source_row < sourceModel()->rowCount() - 1
+            && sourceModel()->data(sourceModel()->index(source_row + 1, 0), MessageEventModel::DelegateTypeRole)
+                != MessageEventModel::DelegateType::State) // If this is the first state in a block, show it. TODO hidden events?
         || sourceModel()->data(sourceModel()->index(source_row, 0), MessageEventModel::ShowSectionRole).toBool(); // If it's a new day, show it
 }
 
@@ -24,6 +24,8 @@ QVariant CollapseStateProxyModel::data(const QModelIndex &index, int role) const
         return stateEventsList(mapToSource(index).row());
     } else if (role == AuthorListRole) {
         return authorList(mapToSource(index).row());
+    } else if (role == ExcessAuthorsRole) {
+        return excessAuthors(mapToSource(index).row());
     }
     return sourceModel()->data(mapToSource(index), role);
 }
@@ -34,6 +36,7 @@ QHash<int, QByteArray> CollapseStateProxyModel::roleNames() const
     roles[AggregateDisplayRole] = "aggregateDisplay";
     roles[StateEventsRole] = "stateEvents";
     roles[AuthorListRole] = "authorList";
+    roles[ExcessAuthorsRole] = "excessAuthors";
     return roles;
 }
 
@@ -47,10 +50,11 @@ QString CollapseStateProxyModel::aggregateEventToString(int sourceRow) const
         if (!uniqueAuthors.contains(nextAuthor)) {
             uniqueAuthors.append(nextAuthor);
         }
-        if (sourceModel()->data(sourceModel()->index(i - 1, 0), MessageEventModel::EventTypeRole)
-                != MessageEventModel::DelegateType::State // If it's not a state event
-            || sourceModel()->data(sourceModel()->index(i - 1, 0), MessageEventModel::ShowSectionRole).toBool() // or the section needs to be visible
-        ) {
+        if (i > 0
+            && (sourceModel()->data(sourceModel()->index(i - 1, 0), MessageEventModel::DelegateTypeRole)
+                    != MessageEventModel::DelegateType::State // If it's not a state event
+                || sourceModel()->data(sourceModel()->index(i - 1, 0), MessageEventModel::ShowSectionRole).toBool() // or the section needs to be visible
+                )) {
             break;
         }
     }
@@ -105,10 +109,11 @@ QVariantList CollapseStateProxyModel::stateEventsList(int sourceRow) const
             {"text", sourceModel()->data(sourceModel()->index(i, 0), Qt::DisplayRole).toString()},
         };
         stateEvents.append(nextState);
-        if (sourceModel()->data(sourceModel()->index(i - 1, 0), MessageEventModel::EventTypeRole)
-                != MessageEventModel::DelegateType::State // If it's not a state event
-            || sourceModel()->data(sourceModel()->index(i - 1, 0), MessageEventModel::ShowSectionRole).toBool() // or the section needs to be visible
-        ) {
+        if (i > 0
+            && (sourceModel()->data(sourceModel()->index(i - 1, 0), MessageEventModel::DelegateTypeRole)
+                    != MessageEventModel::DelegateType::State // If it's not a state event
+                || sourceModel()->data(sourceModel()->index(i - 1, 0), MessageEventModel::ShowSectionRole).toBool() // or the section needs to be visible
+                )) {
             break;
         }
     }
@@ -123,12 +128,50 @@ QVariantList CollapseStateProxyModel::authorList(int sourceRow) const
         if (!uniqueAuthors.contains(nextAvatar)) {
             uniqueAuthors.append(nextAvatar);
         }
-        if (sourceModel()->data(sourceModel()->index(i - 1, 0), MessageEventModel::EventTypeRole)
-                != MessageEventModel::DelegateType::State // If it's not a state event
-            || sourceModel()->data(sourceModel()->index(i - 1, 0), MessageEventModel::ShowSectionRole).toBool() // or the section needs to be visible
-        ) {
+        if (i > 0
+            && (sourceModel()->data(sourceModel()->index(i - 1, 0), MessageEventModel::DelegateTypeRole)
+                    != MessageEventModel::DelegateType::State // If it's not a state event
+                || sourceModel()->data(sourceModel()->index(i - 1, 0), MessageEventModel::ShowSectionRole).toBool() // or the section needs to be visible
+                )) {
             break;
         }
     }
+
+    if (uniqueAuthors.count() > 5) {
+        uniqueAuthors = uniqueAuthors.mid(0, 5);
+    }
     return uniqueAuthors;
 }
+
+QString CollapseStateProxyModel::excessAuthors(int row) const
+{
+    QVariantList uniqueAuthors;
+    for (int i = row; i >= 0; i--) {
+        QVariant nextAvatar = sourceModel()->data(sourceModel()->index(i, 0), MessageEventModel::AuthorRole);
+        if (!uniqueAuthors.contains(nextAvatar)) {
+            uniqueAuthors.append(nextAvatar);
+        }
+        if (i > 0
+            && (sourceModel()->data(sourceModel()->index(i - 1, 0), MessageEventModel::DelegateTypeRole)
+                    != MessageEventModel::DelegateType::State // If it's not a state event
+                || sourceModel()->data(sourceModel()->index(i - 1, 0), MessageEventModel::ShowSectionRole).toBool() // or the section needs to be visible
+                )) {
+            break;
+        }
+    }
+
+    int excessAuthors;
+    if (uniqueAuthors.count() > 5) {
+        excessAuthors = uniqueAuthors.count() - 5;
+    } else {
+        excessAuthors = 0;
+    }
+    QString excessAuthorsString;
+    if (excessAuthors == 0) {
+        return QString();
+    } else {
+        return QStringLiteral("+ %1").arg(excessAuthors);
+    }
+}
+
+#include "moc_collapsestateproxymodel.cpp"
