@@ -2,12 +2,15 @@
 // SPDX-License-Identifier: LGPL-2.0-or-later
 
 #include "pollhandler.h"
+
+#include "events/pollevent.h"
 #include "neochatroom.h"
-#include "pollevent.h"
+
+#include <Quotient/csapi/relations.h>
+#include <Quotient/events/roompowerlevelsevent.h>
+#include <Quotient/user.h>
+
 #include <algorithm>
-#include <csapi/relations.h>
-#include <events/roompowerlevelsevent.h>
-#include <user.h>
 
 using namespace Quotient;
 
@@ -34,9 +37,12 @@ void PollHandler::setRoom(NeoChatRoom *room)
     connect(room, &NeoChatRoom::aboutToAddNewMessages, this, [this](Quotient::RoomEventsRange events) {
         for (const auto &event : events) {
             if (event->is<PollEndEvent>()) {
-                auto pl = m_room->getCurrentState<RoomPowerLevelsEvent>();
-                auto userPl = pl->powerLevelForUser(event->senderId());
-                if (event->senderId() == (*m_room->findInTimeline(m_pollStartEventId))->senderId() || userPl >= pl->redact()) {
+                auto plEvent = m_room->currentState().get<RoomPowerLevelsEvent>();
+                if (!plEvent) {
+                    continue;
+                }
+                auto userPl = plEvent->powerLevelForUser(event->senderId());
+                if (event->senderId() == (*m_room->findInTimeline(m_pollStartEventId))->senderId() || userPl >= plEvent->redact()) {
                     m_hasEnded = true;
                     m_endedTimestamp = event->originTimestamp();
                     Q_EMIT hasEndedChanged();
@@ -75,9 +81,12 @@ void PollHandler::checkLoadRelations()
     connect(job, &BaseJob::success, this, [this, job]() {
         for (const auto &event : job->chunk()) {
             if (event->is<PollEndEvent>()) {
-                auto pl = m_room->getCurrentState<RoomPowerLevelsEvent>();
-                auto userPl = pl->powerLevelForUser(event->senderId());
-                if (event->senderId() == (*m_room->findInTimeline(m_pollStartEventId))->senderId() || userPl >= pl->redact()) {
+                auto plEvent = m_room->currentState().get<RoomPowerLevelsEvent>();
+                if (!plEvent) {
+                    continue;
+                }
+                auto userPl = plEvent->powerLevelForUser(event->senderId());
+                if (event->senderId() == (*m_room->findInTimeline(m_pollStartEventId))->senderId() || userPl >= plEvent->redact()) {
                     m_hasEnded = true;
                     m_endedTimestamp = event->originTimestamp();
                     Q_EMIT hasEndedChanged();
@@ -163,3 +172,5 @@ int PollHandler::answerCount() const
 {
     return m_answers.size();
 }
+
+#include "moc_pollhandler.cpp"
