@@ -3,10 +3,10 @@
 
 #include "spacehierarchycache.h"
 
+#include <Quotient/csapi/space_hierarchy.h>
+#include <Quotient/qt_connection_util.h>
+
 #include "controller.h"
-#ifdef QUOTIENT_07
-#include <csapi/space_hierarchy.h>
-#endif
 #include "neochatroom.h"
 
 using namespace Quotient;
@@ -17,31 +17,31 @@ SpaceHierarchyCache::SpaceHierarchyCache(QObject *parent)
     cacheSpaceHierarchy();
     connect(&Controller::instance(), &Controller::activeConnectionChanged, this, [this]() {
         cacheSpaceHierarchy();
+        connect(Controller::instance().activeConnection(), &Connection::joinedRoom, this, &SpaceHierarchyCache::addSpaceToHierarchy);
+        connect(Controller::instance().activeConnection(), &Connection::aboutToDeleteRoom, this, &SpaceHierarchyCache::removeSpaceFromHierarchy);
     });
 }
 
 void SpaceHierarchyCache::cacheSpaceHierarchy()
 {
-#ifdef QUOTIENT_07
     auto connection = Controller::instance().activeConnection();
     if (!connection) {
         return;
     }
 
-    const auto roomList = connection->allRooms();
+    const auto &roomList = connection->allRooms();
     for (const auto &room : roomList) {
         const auto neoChatRoom = static_cast<NeoChatRoom *>(room);
         if (neoChatRoom->isSpace()) {
             populateSpaceHierarchy(neoChatRoom->id());
         } else {
-            connect(neoChatRoom, &Room::baseStateLoaded, neoChatRoom, [this, neoChatRoom]() {
+            connectSingleShot(neoChatRoom, &Room::baseStateLoaded, neoChatRoom, [this, neoChatRoom]() {
                 if (neoChatRoom->isSpace()) {
                     populateSpaceHierarchy(neoChatRoom->id());
                 }
             });
         }
     }
-#endif
 }
 
 void SpaceHierarchyCache::populateSpaceHierarchy(const QString &spaceId)
@@ -50,8 +50,7 @@ void SpaceHierarchyCache::populateSpaceHierarchy(const QString &spaceId)
     if (!connection) {
         return;
     }
-#ifdef QUOTIENT_07
-    GetSpaceHierarchyJob *job = connection->callApi<GetSpaceHierarchyJob>(spaceId);
+    auto job = connection->callApi<GetSpaceHierarchyJob>(spaceId);
 
     connect(job, &BaseJob::success, this, [this, job, spaceId]() {
         const auto rooms = job->rooms();
@@ -60,12 +59,28 @@ void SpaceHierarchyCache::populateSpaceHierarchy(const QString &spaceId)
             for (const auto &state : rooms[i].childrenState) {
                 roomList.push_back(state->stateKey());
             }
-            roomList.push_back(rooms.at(i).roomId);
         }
         m_spaceHierarchy.insert(spaceId, roomList);
         Q_EMIT spaceHierarchyChanged();
     });
-#endif
+}
+
+void SpaceHierarchyCache::addSpaceToHierarchy(Quotient::Room *room)
+{
+    connectSingleShot(room, &Quotient::Room::baseStateLoaded, this, [this, room]() {
+        const auto neoChatRoom = static_cast<NeoChatRoom *>(room);
+        if (neoChatRoom->isSpace()) {
+            populateSpaceHierarchy(neoChatRoom->id());
+        }
+    });
+}
+
+void SpaceHierarchyCache::removeSpaceFromHierarchy(Quotient::Room *room)
+{
+    const auto neoChatRoom = static_cast<NeoChatRoom *>(room);
+    if (neoChatRoom->isSpace()) {
+        m_spaceHierarchy.remove(neoChatRoom->id());
+    }
 }
 
 QVector<QString> &SpaceHierarchyCache::getRoomListForSpace(const QString &spaceId, bool updateCache)
@@ -75,3 +90,16 @@ QVector<QString> &SpaceHierarchyCache::getRoomListForSpace(const QString &spaceI
     }
     return m_spaceHierarchy[spaceId];
 }
+
+bool SpaceHierarchyCache::isChildSpace(const QString &spaceId) const
+{
+    const auto childrens = m_spaceHierarchy.values();
+    for (const auto &children : childrens) {
+        if (children.contains(spaceId)) {
+            return true;
+        }
+    }
+    return false;
+}
+
+#include "moc_spacehierarchycache.cpp"
